@@ -1,39 +1,38 @@
 /************************************************************************
+ 
+ IMPORTANT NOTE : this file contains two clearly delimited sections :
+ the ARCHITECTURE section (in two parts) and the USER section. Each section
+ is governed by its own copyright and license. Please check individually
+ each section for license and copyright information.
+ *************************************************************************/
 
-    IMPORTANT NOTE : this file contains two clearly delimited sections :
-    the ARCHITECTURE section (in two parts) and the USER section. Each section
-    is governed by its own copyright and license. Please check individually
-    each section for license and copyright information.
-*************************************************************************/
-
-/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/
-
+/******************* BEGIN max-msp.cpp ****************/
 /************************************************************************
-    FAUST Architecture File
-    Copyright (C) 2004-2018 GRAME, Centre National de Creation Musicale
-    ---------------------------------------------------------------------
-    This Architecture section is free software; you can redistribute it
-    and/or modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either version 3
-    of the License, or (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; If not, see <http://www.gnu.org/licenses/>.
-
-    EXCEPTION : As a special exception, you may create a larger work
-    that contains this FAUST architecture section and distribute
-    that work under terms of your choice, so long as this FAUST
-    architecture section is not modified.
-
-    MAX MSP SDK : in order to compile a MaxMSP external with this
-    architecture file you will need the official MaxMSP SDK from
-    cycling'74. Please check the corresponding license.
-
+ FAUST Architecture File
+ Copyright (C) 2004-2018 GRAME, Centre National de Creation Musicale
+ ---------------------------------------------------------------------
+ This Architecture section is free software; you can redistribute it
+ and/or modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either version 3
+ of the License, or (at your option) any later version.
+ 
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU Lesser General Public License for more details.
+ 
+ You should have received a copy of the GNU Lesser General Public License
+ along with this program; If not, see <http://www.gnu.org/licenses/>.
+ 
+ EXCEPTION : As a special exception, you may create a larger work
+ that contains this FAUST architecture section and distribute
+ that work under terms of your choice, so long as this FAUST
+ architecture section is not modified.
+ 
+ MAX MSP SDK : in order to compile a MaxMSP external with this
+ architecture file you will need the official MaxMSP SDK from
+ cycling'74. Please check the corresponding license.
+ 
  ************************************************************************
  ************************************************************************/
 
@@ -132,7 +131,7 @@ using namespace std;
 #define ASSIST_INLET 	1
 #define ASSIST_OUTLET 	2
 
-#define EXTERNAL_VERSION    "0.77"
+#define EXTERNAL_VERSION    "0.87"
 #define STR_SIZE            512
 
 #include "faust/gui/GUI.h"
@@ -140,7 +139,7 @@ using namespace std;
 #include "faust/gui/mspUI.h"
 #include "faust/dsp/poly-dsp.h"
 
-std::list<GUI*> GUI::fGuiList;
+list<GUI*> GUI::fGuiList;
 ztimedmap GUI::gTimedZoneMap;
 
 static t_class* faust_class;
@@ -156,7 +155,6 @@ typedef struct faust
     void** m_args;
     mspUI* m_dspUI;
     dsp* m_dsp;
-    mydsp_poly* m_dsp_poly;
     void* m_control_outlet;
     char* m_json;
     t_systhread_mutex m_mutex;    
@@ -165,7 +163,8 @@ typedef struct faust
     SaveUI* m_savedUI;
 #ifdef MIDICTRL
     MidiUI* m_midiUI;
-    midi_handler* m_midiHandler;
+    max_midi* m_midiHandler;
+    void* m_midi_outlet;
 #endif
 #ifdef SOUNDFILE
     SoundUI* m_soundInterface;
@@ -183,57 +182,41 @@ void faust_allocate(t_faust* x, int nvoices)
 {
     // Delete old
     delete x->m_dsp;
-    if (x->m_dspUI) x->m_dspUI->clear();
+    delete x->m_dspUI;
+    // Preserve and restore saved UI state across reallocations
+    if (x->m_savedUI) {
+        x->m_savedUI->save();
+    }
+    x->m_dspUI = new mspUI();
     
     if (nvoices > 0) {
+    #ifdef POST
         post("polyphonic DSP voices = %d", nvoices);
-        x->m_dsp_poly = new mydsp_poly(new mydsp(), nvoices, true, true);
-#ifdef POLY2
-        x->m_dsp = new dsp_sequencer(x->m_dsp_poly, new effect());
-#else
-        x->m_dsp = x->m_dsp_poly;
-#endif
-#ifdef MIDICTRL
-        x->m_midiHandler->addMidiIn(x->m_dsp_poly);
-#endif
+    #endif
+        x->m_dsp = new mydsp_poly(new mydsp(), nvoices, true, true);
+    #ifdef POLY2
+        x->m_dsp = new dsp_sequencer(x->m_dsp, new effect());
+    #endif
     } else {
+    #ifdef POST
         post("monophonic DSP");
-        #if (DOWN_SAMPLING > 0)
-            #if (FILTER_TYPE == 0)
-                x->m_dsp = new dsp_down_sampler<Identity<Double<1,1>, DOWN_SAMPLING>>(new mydsp());
-            #elif (FILTER_TYPE == 1)
-                x->m_dsp = new dsp_down_sampler<LowPass3<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 2)
-                x->m_dsp = new dsp_down_sampler<LowPass4<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 3)
-                x->m_dsp = new dsp_down_sampler<LowPass3e<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 4)
-                x->m_dsp = new dsp_down_sampler<LowPass6eé<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-            #else
-                #error "ERROR : Filter type must be in [0..4] range"
-            #endif
-        #elif (UP_SAMPLING > 0)
-            #if (FILTER_TYPE == 0)
-                x->m_dsp = new dsp_up_sampler<Identity<Double<1,1>, UP_SAMPLING>>(new mydsp());
-            #elif (FILTER_TYPE == 1)
-                x->m_dsp = new dsp_up_sampler<LowPass3<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 2)
-                x->m_dsp = new dsp_up_sampler<LowPass4<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 3)
-                x->m_dsp = new dsp_up_sampler<LowPass3e<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #elif (FILTER_TYPE == 4)
-                x->m_dsp = new dsp_up_sampler<LowPass6e<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-            #else
-                #error "ERROR : Filter type must be in [0..4] range"
-            #endif
-        #else
-            x->m_dsp = new mydsp();
-        #endif
+    #endif
+        // Create a DS/US + Filter adapted DSP
+        std::string error;
+        x->m_dsp = createSRAdapter<FAUSTFLOAT>(new mydsp(), error, DOWN_SAMPLING, UP_SAMPLING, FILTER_TYPE);
     }
     
 #ifdef MIDICTRL
     x->m_dsp->buildUserInterface(x->m_midiUI);
 #endif
+    // Restore saved UI to new DSP mapping
+    if (x->m_savedUI) {
+        x->m_dsp->buildUserInterface(x->m_savedUI);
+        x->m_savedUI->load();
+    }
+    // Cache current IO counts
+    x->m_Inputs = x->m_dsp->getNumInputs();
+    x->m_Outputs = x->m_dsp->getNumOutputs();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -250,9 +233,11 @@ void faust_anything(t_faust* obj, t_symbol* s, short ac, t_atom* av)
         obj->m_dspUI->setValue(name, off);
         obj->m_dspUI->setValue(name, on);
         
-        av[0].a_type = A_FLOAT;
-        av[0].a_w.w_float = off;
-        faust_anything(obj, s, 1, av);
+        // Safe toggle using a local atom (ac can be 0, so av may be nullptr)
+        t_atom toggle_atom[1];
+        toggle_atom[0].a_type = A_FLOAT;
+        toggle_atom[0].a_w.w_float = off;
+        faust_anything(obj, s, 1, toggle_atom);
         
     } else if (mspUI::checkDigit(name)) { // List of values
         
@@ -309,10 +294,14 @@ void faust_anything(t_faust* obj, t_symbol* s, short ac, t_atom* av)
         
     } else {
         // Standard parameter name
-        FAUSTFLOAT value = (av[0].a_type == A_LONG) ? FAUSTFLOAT(av[0].a_w.w_long) : FAUSTFLOAT(av[0].a_w.w_float);
-        res = obj->m_dspUI->setValue(name, value);
-        if (!res) {
-            post("Unknown parameter : %s", (s)->s_name);
+        if (ac > 0) {
+            FAUSTFLOAT value = (av[0].a_type == A_LONG) ? FAUSTFLOAT(av[0].a_w.w_long) : FAUSTFLOAT(av[0].a_w.w_float);
+            res = obj->m_dspUI->setValue(name, value);
+            if (!res) {
+                post("Unknown parameter : %s", (s)->s_name);
+            }
+        } else {
+            post("No value provided for parameter : %s", (s)->s_name);
         }
     }
 }
@@ -320,31 +309,12 @@ void faust_anything(t_faust* obj, t_symbol* s, short ac, t_atom* av)
 /*--------------------------------------------------------------------------*/
 void faust_polyphony(t_faust* x, t_symbol* s, short ac, t_atom* av)
 {
+    if (ac < 1 || atom_gettype(&av[0]) != A_LONG) {
+        post("polyphony requires a long argument (voices)");
+        return;
+    }
     if (systhread_mutex_lock(x->m_mutex) == MAX_ERR_NONE) {
-        
-    #ifdef MIDICTRL
-        mydsp_poly* poly = dynamic_cast<mydsp_poly*>(x->m_dsp);
-        if (poly) {
-            x->m_midiHandler->removeMidiIn(poly);
-        }
-    #endif
-        
         faust_allocate(x, av[0].a_w.w_long);
-        
-        // Initialize at the system's sampling rate
-        x->m_dsp->init(long(sys_getsr()));
-        // Initialize User Interface (here connnection with controls)
-        x->m_dsp->buildUserInterface(x->m_dspUI);
-        
-        // Prepare JSON
-        faust_make_json(x);
-      
-        // Send JSON to JS script
-        faust_create_jsui(x);
-        
-        // Load old controller state
-        x->m_dsp->buildUserInterface(x->m_savedUI);
-        
         systhread_mutex_unlock(x->m_mutex);
     } else {
         post("Mutex lock cannot be taken...");
@@ -401,13 +371,13 @@ void faust_create_jsui(t_faust* x)
 /*--------------------------------------------------------------------------*/
 void faust_update_outputs(t_faust* x)
 {
-    for (auto& it1 : x->m_output_table) {
+    for (const auto& it1 : x->m_output_table) {
         bool new_val = false;
         FAUSTFLOAT value = x->m_dspUI->getOutputValue(it1.first, new_val);
         if (new_val) {
             t_atom at_value;
             atom_setfloat(&at_value, value);
-            for (auto& it2 : it1.second) {
+            for (const auto& it2 : it1.second) {
                 object_method_typed(it2, gensym("float"), 1, &at_value, 0);
             }
         }
@@ -429,88 +399,70 @@ void faust_make_json(t_faust* x)
 void* faust_new(t_symbol* s, short ac, t_atom* av)
 {
     bool midi_sync = false;
+    bool midi = false;
     int nvoices = 0;
-    
-    mydsp* tmp_dsp = new mydsp();
-    MidiMeta::analyse(tmp_dsp, midi_sync, nvoices);
-    delete tmp_dsp;
- 
     t_faust* x = (t_faust*)object_alloc(faust_class);
     
-    x->m_savedUI = new SaveLabelUI();
-    x->m_dspUI = NULL;
-    x->m_dsp = NULL;
-    x->m_dsp_poly = NULL;
-    x->m_json = NULL;
-    x->m_mute = false;
-    
-#ifdef MIDICTRL
-    x->m_midiHandler = new midi_handler();
-    x->m_midiUI = new MidiUI(x->m_midiHandler);
-#endif
-
-    faust_allocate(x, nvoices);
-
-    x->m_Inputs = x->m_dsp->getNumInputs();
-    x->m_Outputs = x->m_dsp->getNumOutputs();
-   
-    x->m_dspUI = new mspUI();
-    x->m_control_outlet = outlet_new((t_pxobject*)x, (char*)"list");
-
-    // Initialize at the system's sampling rate
-    x->m_dsp->init(long(sys_getsr()));
-    // Initialize User Interface (here connnection with controls)
-    x->m_dsp->buildUserInterface(x->m_dspUI);
-    
-    t_max_err err = systhread_mutex_new(&x->m_mutex, SYSTHREAD_MUTEX_NORMAL);
-    if (err != MAX_ERR_NONE) {
-        post("Cannot allocate mutex...");
-    }
-    
-    // Prepare JSON
-    faust_make_json(x);
-    
-    x->m_args = (void**)calloc((x->m_dsp->getNumInputs() + x->m_dsp->getNumOutputs()) + 2, sizeof(void*));
-   /* Multi in */
-    dsp_setup((t_pxobject*)x, x->m_dsp->getNumInputs());
-
-    /* Multi out */
-    for (int i = 0; i < x->m_dsp->getNumOutputs(); i++) {
-        outlet_new((t_pxobject*)x, (char*)"signal");
-    }
-
-    ((t_pxobject*)x)->z_misc = Z_NO_INPLACE; // To assure input and output buffers are actually different
-    
+    mydsp* tmp_dsp = new mydsp();
+    MidiMeta::analyse(tmp_dsp, midi, midi_sync, nvoices);
 #ifdef SOUNDFILE
     Max_Meta3 meta3;
-    x->m_dsp->metadata(&meta3);
+    tmp_dsp->metadata(&meta3);
     string bundle_path_str = SoundUI::getBinaryPathFrom(meta3.fName);
     if (bundle_path_str == "") {
         post("Bundle_path '%s' cannot be found!", meta3.fName.c_str());
     }
-    x->m_soundInterface = new SoundUI(bundle_path_str);
-    // SoundUI has to be dispatched on all internal voices
-    if (x->m_dsp_poly) x->m_dsp_poly->setGroup(false);
-    x->m_dsp->buildUserInterface(x->m_soundInterface);
-    if (x->m_dsp_poly) x->m_dsp_poly->setGroup(true);
+    x->m_soundInterface = new SoundUI(bundle_path_str, -1, nullptr, sizeof(FAUSTFLOAT) == 8);
+#endif
+    delete tmp_dsp;
+    
+    x->m_savedUI = new SaveLabelUI();
+    // allocation is done in faust_allocate
+    x->m_dspUI = NULL;
+    x->m_dsp = NULL;
+    x->m_json = NULL;
+    x->m_mute = false;
+    x->m_control_outlet = outlet_new((t_pxobject*)x, (char*)"list");
+    
+#ifdef MIDICTRL
+    x->m_midi_outlet = outlet_new((t_pxobject*)x, NULL);
+    x->m_midiHandler = new max_midi(x->m_midi_outlet);
+    x->m_midiUI = new MidiUI(x->m_midiHandler);
 #endif
     
-#ifdef OSCCTRL
-    x->m_oscInterface = NULL;
-#endif
+    faust_allocate(x, nvoices);
     
-    // Send JSON to JS script
-    faust_create_jsui(x);
+    if (systhread_mutex_new(&x->m_mutex, SYSTHREAD_MUTEX_NORMAL) != MAX_ERR_NONE) {
+        post("Cannot allocate mutex...");
+    }
     
-    // Load old controller state
-    x->m_dsp->buildUserInterface(x->m_savedUI);
+    int num_input;
+    if (x->m_dspUI->isMulti()) {
+        num_input = x->m_dsp->getNumInputs() + 1;
+    } else {
+        num_input = x->m_dsp->getNumInputs();
+    }
+    
+    free(x->m_args);
+    x->m_args = (void**)calloc((num_input + x->m_dsp->getNumOutputs()) + 2, sizeof(void*));
+    
+    /* Multi in */
+    dsp_setup((t_pxobject*)x, num_input);
+    
+    /* Multi out */
+    for (int i = 0; i < x->m_dsp->getNumOutputs(); i++) {
+        outlet_new((t_pxobject*)x, (char*)"signal");
+    }
+    
+    ((t_pxobject*)x)->z_misc = Z_NO_INPLACE; // To assure input and output buffers are actually different
     
     // Display controls
+#ifdef POST
     x->m_dspUI->displayControls();
+#endif
     
     // Get attributes values
     attr_args_process(x, ac, av);
-    
     return x;
 }
 
@@ -520,6 +472,14 @@ void* faust_new(t_symbol* s, short ac, t_atom* av)
 void faust_osc(t_faust* x, t_symbol* s, short ac, t_atom* av)
 {
     if (ac == 5) {
+        if (atom_gettype(&av[0]) != A_SYM
+            || atom_gettype(&av[1]) != A_LONG
+            || atom_gettype(&av[2]) != A_LONG
+            || atom_gettype(&av[3]) != A_LONG
+            || atom_gettype(&av[4]) != A_LONG) {
+            post("Should be : osc 'IP inport outport xmit(0|1|2) bundle(0|1)'");
+            return;
+        }
         if (systhread_mutex_lock(x->m_mutex) == MAX_ERR_NONE) {
             
             delete x->m_oscInterface;
@@ -595,8 +555,7 @@ void faust_init(t_faust* x, t_symbol* s, short ac, t_atom* av)
 }
 
 /*--------------------------------------------------------------------------*/
-// Dump controllers as list of: [path, cur, min, max]
-void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
+void faust_dump_inputs(t_faust* x)
 {
     // Input controllers
     for (mspUI::iterator it = x->m_dspUI->begin2(); it != x->m_dspUI->end2(); it++) {
@@ -607,6 +566,11 @@ void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
         atom_setfloat(&myList[3], (*it).second->getMaxValue());
         outlet_list(x->m_control_outlet, 0, 4, myList);
     }
+}
+
+/*--------------------------------------------------------------------------*/
+void faust_dump_outputs(t_faust* x)
+{
     // Output controllers
     for (mspUI::iterator it = x->m_dspUI->begin4(); it != x->m_dspUI->end4(); it++) {
         t_atom myList[4];
@@ -618,6 +582,14 @@ void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
     }
 }
 
+/*--------------------------------------------------------------------------*/
+// Dump controllers as list of [path, cur, min, max]
+void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
+{
+    faust_dump_inputs(x);
+    faust_dump_outputs(x);
+}
+
 /*--------------------------------------------------------------------------*/
 void faust_dblclick(t_faust* x, long inlet)
 {
@@ -631,18 +603,20 @@ void faust_assist(t_faust* x, void* b, long msg, long a, char* dst)
     if (msg == ASSIST_INLET) {
         if (a == 0) {
             if (x->m_dsp->getNumInputs() == 0) {
-                sprintf(dst, "(message) : Unused Input");
+                snprintf(dst, 512, "(messages)");
             } else {
-                sprintf(dst, "(signal) : Audio Input %ld", (a+1));
+                snprintf(dst, 512, "(messages/signal) : Audio Input %ld", (a+1));
             }
         } else if (a < x->m_dsp->getNumInputs()) {
-            sprintf(dst, "(signal) : Audio Input %ld", (a+1));
+            snprintf(dst, 512, "(signal) : Audio Input %ld", (a+1));
         }
     } else if (msg == ASSIST_OUTLET) {
         if (a < x->m_dsp->getNumOutputs()) {
-            sprintf(dst, "(signal) : Audio Output %ld", (a+1));
+            snprintf(dst, 512, "(signal) : Audio Output %ld", (a+1));
+        } else if (a == x->m_dsp->getNumOutputs()) {
+            snprintf(dst, 512, "(list) : [path, cur|init, min, max]*");
         } else {
-            sprintf(dst, "(list) : [path, cur|init, min, max]*");
+            snprintf(dst, 512, "(int) : raw MIDI bytes*");
         }
     }
 }
@@ -659,22 +633,22 @@ void faust_mute(t_faust* obj, t_symbol* s, short ac, t_atom* at)
 void faust_free(t_faust* x)
 {
     dsp_free((t_pxobject*)x);
-    delete x->m_dsp;
-    delete x->m_dspUI;
-    delete x->m_savedUI;
-    if (x->m_args) free(x->m_args);
-    if (x->m_json) free(x->m_json);
-    systhread_mutex_free(x->m_mutex);
+    delete x->m_dsp;      x->m_dsp = nullptr;
+    delete x->m_dspUI;    x->m_dspUI = nullptr;
+    delete x->m_savedUI;  x->m_savedUI = nullptr;
+    if (x->m_args) { free(x->m_args); x->m_args = nullptr; }
+    if (x->m_json) { free(x->m_json); x->m_json = nullptr; }
+    systhread_mutex_free(x->m_mutex); x->m_mutex = nullptr;
 #ifdef MIDICTRL
     // m_midiUI *must* be deleted before m_midiHandler
-    delete x->m_midiUI;
-    delete x->m_midiHandler;
+    delete x->m_midiUI;    x->m_midiUI = nullptr;
+    delete x->m_midiHandler; x->m_midiHandler = nullptr;
 #endif
 #ifdef SOUNDFILE
-    delete x->m_soundInterface;
+    delete x->m_soundInterface; x->m_soundInterface = nullptr;
 #endif
 #ifdef OSCCTRL
-    delete x->m_oscInterface;
+    delete x->m_oscInterface; x->m_oscInterface = nullptr;
 #endif
 }
 
@@ -688,10 +662,9 @@ t_int* faust_perform(t_int* w)
     if (!x->m_mute && systhread_mutex_trylock(x->m_mutex) == MAX_ERR_NONE) {
         if (x->m_dsp) {
             x->m_dsp->compute(n, ((float**)&w[offset]), ((float**)&w[offset + x->m_dsp->getNumInputs()]));
-        #ifdef OSCCTRL
-            if (x->m_oscInterface) x->m_oscInterface->endBundle();
-        #endif
-            faust_update_outputs(x);
+            // faust_update_outputs(x);
+            // Use the right outlet to output messages
+            faust_dump_outputs(x);
         }
     #if defined(MIDICTRL) || defined(OSCCTRL)
         GUI::updateAllGuis();
@@ -761,8 +734,7 @@ void ext_main(void* r)
     tmp_dsp->buildUserInterface(&tmp_UI);
     
     // Setup attribute
-    int i = 0;
-    for (mspUI::iterator it = tmp_UI.begin1(); it != tmp_UI.end1(); it++, i++) {
+    for (mspUI::iterator it = tmp_UI.begin1(); it != tmp_UI.end1(); it++) {
         CLASS_ATTR_FLOAT(c, (*it).first.c_str(), 0, t_faust, m_ob);
         CLASS_ATTR_ACCESSORS(c, (*it).first.c_str(), NULL, (method)faust_attr_set);
     }
@@ -770,10 +742,10 @@ void ext_main(void* r)
     class_dspinit(c);
     class_register(CLASS_BOX, c);
     faust_class = c;
-    
+#ifdef POST
     post((char*)"Faust DSP object v%s (sample = 32 bits code = 32 bits)", EXTERNAL_VERSION);
-    post((char*)"Copyright (c) 2012-2020 Grame");
-   
+    post((char*)"Copyright (c) 2012-2025 Grame");
+#endif
     Max_Meta1 meta1;
     tmp_dsp->metadata(&meta1);
     if (meta1.fCount > 0) {
@@ -788,5 +760,4 @@ void ext_main(void* r)
 #endif
 }
 
-/********************END ARCHITECTURE SECTION (part 2/2)****************/
-
+/******************* END max-msp.cpp ****************/
